from __future__ import unicode_literals

from tunigo import translator


def set_instance_variables(instance, keys, values, convert):
    for key_underscore in keys:
        key_camelcase = translator.underscore_to_camelcase(key_underscore)
        if key_camelcase in values:
            setattr(instance, key_underscore, convert(values[key_camelcase]))
        else:
            setattr(instance, key_underscore, convert(None))


def set_instance_array_variables(instance, keys, values):
    set_instance_variables(instance, keys, values, lambda x: x or [])


def set_instance_int_variables(instance, keys, values):
    set_instance_variables(instance, keys, values,
                           lambda x: int(x) if x else 0)


def set_instance_string_variables(instance, keys, values):
    set_instance_variables(instance, keys, values, lambda x: x or '')



from __future__ import unicode_literals


underscore_to_camelcase_cache = {}


def underscore_to_camelcase(word):
    if word in underscore_to_camelcase_cache:
        return underscore_to_camelcase_cache[word]
    else:
        s = ''.join(part.capitalize() for part in word.split('_'))
        s = s[0].lower() + s[1:]
        underscore_to_camelcase_cache[word] = s
        return s



from __future__ import unicode_literals

import tunigo
from tunigo import utils


class Genre(object):

    def __init__(self,
                 created=0,
                 header_image_url='',
                 icon_image_url='',
                 icon_url='',
                 id='',
                 location='',
                 mood_image_url='',
                 name='',
                 number_playlists=0,
                 playlist=None,
                 playlist_uri='',
                 sub_genres=[],
                 template_name='',
                 type='',
                 updated=0,
                 version=0,
                 item_array=None):

        if item_array:
            utils.set_instance_int_variables(
                self,
                ['_created', '_updated', '_version'],
                item_array)
            utils.set_instance_string_variables(
                self,
                ['_header_image_url', '_icon_image_url', '_icon_url', '_id',
                 '_location', '_mood_image_url', '_name', '_number_playlists',
                 '_template_name', '_type'],
                item_array)

            if 'playlistUri' in item_array:
                self._playlist = tunigo.Playlist(main_genre=self,
                                                 uri=item_array['playlistUri'])
            else:
                self._playlist = tunigo.Playlist()

            self._sub_genres = []
            if 'subGenres' in item_array:
                for sub_genre in item_array['subGenres']:
                    self._sub_genres.append(SubGenre(key=sub_genre['key'],
                                                     main_genre=self,
                                                     name=sub_genre['name']))

        else:
            self._created = int(created)
            self._header_image_url = header_image_url
            self._icon_image_url = icon_image_url
            self._icon_url = icon_url
            self._id = id
            self._location = location
            self._mood_image_url = mood_image_url
            self._name = name
            self._number_playlists = int(number_playlists)

            if playlist_uri:
                self._playlist = tunigo.Playlist(main_genre=self,
                                                 uri=playlist_uri)
            else:
                self._playlist = playlist

            self._sub_genres = sub_genres
            self._template_name = template_name
            self._type = type
            self._updated = int(updated)
            self._version = int(version)

    def __repr__(self):
        return "Genre(template_name='{}')".format(self._template_name)

    def __str__(self):
        return self._template_name

    @property
    def created(self):
        return self._created

    @property
    def header_image_url(self):
        return self._header_image_url

    @property
    def icon_image_url(self):
        return self._icon_image_url

    @property
    def icon_url(self):
        return self._icon_url

    @property
    def id(self):
        return self._id

    @property
    def key(self):
        return self._template_name

    @property
    def location(self):
        return self._location

    @property
    def mood_image_url(self):
        return self._mood_image_url

    @property
    def name(self):
        return self._name

    @property
    def number_playlists(self):
        return self._number_playlists

    @property
    def playlist(self):
        return self._playlist

    @property
    def playlist_uri(self):
        return self._playlist.uri

    @property
    def sub_genres(self):
        return self._sub_genres

    @property
    def template_name(self):
        return self._template_name

    @property
    def type(self):
        return self._type

    @property
    def updated(self):
        return self._updated

    @property
    def version(self):
        return self._version


class SubGenre(object):

    def __init__(self, key='', main_genre=None, name=''):
        self._key = key
        if type(main_genre) == Genre:
            self._main_genre = main_genre
        else:
            self._main_genre = Genre(template_name=main_genre)
        self._name = name

    def __repr__(self):
        return ("SubGenre(main_genre='{}', key='{}')"
                .format(self._main_genre, self._key))

    def __str__(self):
        return '{}/{}'.format(self._main_genre, self._key)

    @property
    def key(self):
        return self._key

    @property
    def main_genre(self):
        return self._main_genre

    @property
    def name(self):
        return self._name



from __future__ import unicode_literals


__version__ = '0.1.2'


from tunigo.api import Tunigo
from tunigo.genre import Genre, SubGenre
from tunigo.playlist import Playlist
from tunigo.release import Release


__all__ = [
    'Genre',
    'Playlist',
    'Release',
    'SubGenre',
    'Tunigo',
]



from __future__ import unicode_literals

import time


class Cache(object):

    def __init__(self, cache_time):
        self._cache = {}
        self._cache_time = cache_time

    def __repr__(self):
        return 'Cache(cache_time={})'.format(self._cache_time)

    def _cache_valid(self, key):
        return (key in self._cache and
                self._cache[key]['access_time'] >
                time.time() - self._cache_time)

    def get(self, key):
        if self._cache_valid(key):
            return self._cache[key]['obj']
        else:
            return False

    def insert(self, key, obj):
        if self._cache_time:
            self._cache[key] = {
                'access_time': time.time(),
                'obj': obj
            }



from __future__ import unicode_literals

from tunigo import utils


class Release(object):

    def __init__(self,
                 album_name='',
                 artist_name='',
                 author_ids=[],
                 created=0,
                 description='',
                 genre_id='',
                 id='',
                 image='',
                 location='',
                 num_tracks=0,
                 publication_date='',
                 regions=[],
                 tags=[],
                 updated=0,
                 uri='',
                 version=0,
                 item_array=None):

        if item_array:
            utils.set_instance_string_variables(
                self,
                ['_album_name', '_artist_name', '_created', '_description',
                 '_genre_id', '_id', '_image', '_location',
                 '_publication_date', '_uri'],
                item_array)
            utils.set_instance_array_variables(
                self,
                ['_author_ids', '_regions', '_tags'],
                item_array)
            utils.set_instance_int_variables(
                self,
                ['_num_tracks', '_updated', '_version'],
                item_array)

        else:
            self._album_name = album_name
            self._artist_name = artist_name
            self._author_ids = author_ids
            self._created = created
            self._description = description
            self._genre_id = genre_id
            self._id = id
            self._image = image
            self._location = location
            self._num_tracks = int(num_tracks)
            self._publication_date = publication_date
            self._regions = regions
            self._tags = tags
            self._updated = int(updated)
            self._uri = uri
            self._version = int(version)

    def __repr__(self):
        return "Release(uri='{}')".format(self._uri)

    def __str__(self):
        return '{} - {} ({})'.format(self._artist_name,
                                     self._album_name,
                                     self._uri)

    @property
    def album_name(self):
        return self._album_name

    @property
    def artist_name(self):
        return self._artist_name

    @property
    def author_ids(self):
        return self._author_ids

    @property
    def created(self):
        return self._created

    @property
    def description(self):
        return self._description

    @property
    def genre_id(self):
        return self._genre_id

    @property
    def id(self):
        return self._id

    @property
    def image(self):
        return self._image

    @property
    def location(self):
        return self._location

    @property
    def num_tracks(self):
        return self._num_tracks

    @property
    def publication_date(self):
        return self._publication_date

    @property
    def regions(self):
        return self._regions

    @property
    def tags(self):
        return self._tags

    @property
    def updated(self):
        return self._updated

    @property
    def uri(self):
        return self._uri

    @property
    def version(self):
        return self._version



from __future__ import unicode_literals

import time

import requests

from tunigo.cache import Cache
from tunigo.genre import Genre, SubGenre
from tunigo.playlist import Playlist
from tunigo.release import Release


BASE_URL = 'https://api.tunigo.com/v3/space'
BASE_QUERY = 'locale=en&product=premium&version=6.38.31&platform=web'


class Tunigo(object):

    def __init__(self, region='all', max_results=1000, cache_time=3600):
        self._region = region
        self._max_results = max_results
        self._cache = Cache(cache_time)

    def __repr__(self):
        return "Tunigo(region='{}', max_results={}, cache_time={})".format(
            self._region,
            self._max_results,
            self._cache._cache_time)

    def _get(self, key, options=''):
        uri = ('{}/{}?region={}&per_page={}&{}'
               .format(BASE_URL, key, self._region,
                       self._max_results, BASE_QUERY))
        if options:
            uri = '{}&{}'.format(uri, options)
        result = requests.get(uri)
        if (result.status_code != 200 or
                result.headers['content-type'] != 'application/json'):
            return []
        return result.json()['items']

    def get_playlists(self, key, options='', cache_key=''):
        if not cache_key:
            cache_key = 'playlists-{}-{}'.format(key, options)
        cache_value = self._cache.get(cache_key)
        if cache_value:
            return cache_value
        else:
            playlists = []
            for item in self._get(key, options):
                playlists.append(Playlist(item_array=item['playlist']))
            self._cache.insert(cache_key, playlists)
            return playlists

    def get_featured_playlists(self):
        return self.get_playlists('featured-playlists',
                                  'dt={}'.format(time.strftime('%FT%H:01:00')),
                                  'featured-playlists')

    def get_top_lists(self):
        return self.get_playlists('toplists')

    def get_genres(self):
        cache_key = 'genres'
        cache_value = self._cache.get(cache_key)
        if cache_value:
            return cache_value
        else:
            genres = []
            for item in self._get('genres'):
                if item['genre']['templateName'] != 'toplists':
                    genres.append(Genre(item_array=item['genre']))
            self._cache.insert(cache_key, genres)
            return genres

    def get_genre_playlists(self, genre=None, sub_genre=None):
        if type(genre) == Genre:
            genre_key = genre.key
        else:
            genre_key = genre
        if type(sub_genre) == SubGenre:
            sub_genre_key = sub_genre.key
            if not genre_key:
                genre_key = sub_genre.main_genre.key
        else:
            sub_genre_key = sub_genre

        if sub_genre_key and sub_genre_key != 'all':
            options = 'filter={}'.format(sub_genre_key)
        else:
            options = ''
        return self.get_playlists(genre_key, options)

    def get_new_releases(self):
        cache_key = 'releases'
        cache_value = self._cache.get(cache_key)
        if cache_value:
            return cache_value
        else:
            releases = []
            for item in self._get('new-releases'):
                releases.append(Release(item_array=item['release']))
            self._cache.insert(cache_key, releases)
            return releases



from __future__ import unicode_literals

import tunigo
from tunigo import utils


class Playlist(object):

    def __init__(self,
                 created=0,
                 description='',
                 id='',
                 image='',
                 location='',
                 main_genre=None,
                 main_genre_template='',
                 num_subscribers=0,
                 sub_genre=None,
                 sub_genre_template='',
                 title='',
                 updated=0,
                 uri='',
                 version=0,
                 item_array=None):

        if item_array:
            utils.set_instance_int_variables(
                self,
                ['_created', '_num_subscribers', '_updated', '_version'],
                item_array)
            utils.set_instance_string_variables(
                self,
                ['_description', '_id', '_image', '_location', '_title',
                 '_uri'],
                item_array)

            if 'mainGenreTemplate' in item_array:
                self._main_genre = tunigo.Genre(
                    playlist=self,
                    template_name=item_array['mainGenreTemplate'])
            else:
                self._main_genre = tunigo.Genre()

            if 'subGenreTemplate' in item_array:
                self._sub_genre = tunigo.SubGenre(
                    key=item_array['subGenreTemplate'],
                    main_genre=self._main_genre)
            else:
                self._sub_genre = tunigo.SubGenre()

        else:
            self._created = int(created)
            self._description = description
            self._id = id
            self._image = image
            self._location = location

            if main_genre_template:
                self._main_genre = tunigo.Genre(
                    playlist=self,
                    template_name=main_genre_template)
            else:
                self._main_genre = main_genre

            self._num_subscribers = int(num_subscribers)

            if sub_genre_template:
                self._sub_genre = tunigo.SubGenre(key=sub_genre_template,
                                                  main_genre=self._main_genre)
            else:
                self._sub_genre = sub_genre

            self._title = title
            self._updated = int(updated)
            self._uri = uri
            self._version = int(version)

    def __repr__(self):
        return "Playlist(uri='{}')".format(self._uri)

    def __str__(self):
        return '{} ({})'.format(self._title, self._uri)

    @property
    def created(self):
        return self._created

    @property
    def description(self):
        return self._description

    @property
    def id(self):
        return self._id

    @property
    def image(self):
        return self._image

    @property
    def location(self):
        return self._location

    @property
    def main_genre(self):
        return self._main_genre

    @property
    def main_genre_template(self):
        return self._main_genre.template_name

    @property
    def num_subscribers(self):
        return self._num_subscribers

    @property
    def sub_genre(self):
        return self._sub_genre

    @property
    def sub_genre_template(self):
        return self._sub_genre.template_name

    @property
    def title(self):
        return self._title

    @property
    def updated(self):
        return self._updated

    @property
    def uri(self):
        return self._uri

    @property
    def version(self):
        return self._version
